/*
BSL-Scripter-3.0.1.0

==BSL_Scripter.cpp==
(main program)

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free, 
non-exclusive license under copyrights and patents it now or 
hereafter owns or controls to make, have made, use, import, 
offer to sell and sell ("Utilize") this software subject to 
the terms herein.  With respect to the foregoing patent license, 
such license is granted  solely to the extent that any such patent 
is necessary to Utilize the software alone.  The patent license 
shall not apply to any combinations which include this software, 
other than combinations with devices manufactured by or for TI 
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and 
reproduce this license (including the above copyright notice 
and the disclaimer and (if applicable) source code license 
limitations below) in the documentation and/or other materials 
provided with the distribution

Redistribution and use in binary form, without modification, 
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of 
    this software is permitted with respect to any software 
	provided in binary form.

*	any redistribution and use are licensed by TI for use only 
    with TI Devices.

*	Nothing shall obligate TI to provide you with source code 
    for the software licensed and provided to you in object code.

If software source code is provided to you, modification and 
redistribution of the source code are permitted provided that 
the following conditions are met:

*	any redistribution and use of the source code, including any 
    resulting derivative works, are licensed by TI for use only 
	with TI Devices.

*	any redistribution and use of any object code compiled from 
    the source code and any resulting derivative works, are licensed 
	by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names 
of its suppliers may be used to endorse or promote products derived 
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND 
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, 
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS 
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, 
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR 
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, 
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/

#include "stdafx.h"
#include "hidapi.h"
#include <stdio.h>


//Global Variables
ScripterStream   scripterStream;
Interpreter      interpreter;
std::string      mainPathDir;
uint8_t*         ptrBigBuf;
std::string      getResponseString;
ExecutionCommand execCommand;
ExecutionCommunication execCommunication;


int main(int argc, char * argv[])
{
	try
	{
		mainPathDir = scripterStream.startScripter(argv[1]);
	}
	catch (std::runtime_error& e)
	{
		cerr << e.what() << endl;
		return (-1);
	}
 
	scripterStream.numOfLines = 0;

	while (scripterStream.getLine())
	{
		interpreter.clearParamsInput();
		
		std::cout << scripterStream.retStringCommand() << endl;
		
		bool blCheckComment = interpreter.checkCommentLine(scripterStream.retStringCommand());

		if (blCheckComment == false)
		{
			++scripterStream.numOfLines;

			if (scripterStream.numOfLines == 1)
			{
				interpreter.interpretCommandParams(scripterStream.retStringCommand());

				try
				{
					interpreter.checkModeParameters();
				}
				catch (std::runtime_error& e)
				{
					cerr << e.what() << endl;
					return (-1);
				}

				execCommunication.setTarget(interpreter.retActualCommandParams());
				execCommunication.init(interpreter.retActualCommandParams());
				execCommand.setTarget(interpreter.retActualCommandParams());
				execCommand.initVerbose();

				//Addition initialization for P432 family
				if (interpreter.retActualCommandParams().family == Family::P4xx)
				{
					const uint8_t loopInitialization = 20;
					bool InitP432 = false;

					for (uint8_t i = 0; i < loopInitialization;i++)
					{
						execCommunication.transmitBufferInit432();
						ptrBigBuf = execCommunication.receiveBuffer(1, execCommand.retVerbose());
						if (*ptrBigBuf == 0x00)
						{
							InitP432 = true;
							break;
						}
					}
					if (InitP432 == false)
					{
						std::cout << "Initialization of BSL P432 failed... Exit the scripter..." << std::endl;
						return(-1);
					}
					else
					{
						std::cout << "Initialization of BSL P432 succeed..." << std::endl;
					}
				}
			}
			else
			{
				Family tempFamily = interpreter.retActualCommandParams().family;
				interpreter.interpretCommandParams(scripterStream.retStringCommand());
				if (interpreter.retActualCommandParams().repeatMode)
				{
					try
					{
						interpreter.checkModeParameters();
						if (tempFamily != interpreter.retActualCommandParams().family)
						{
							cerr << "Family of device does not match the previous setting...";
							return (-1);
						}
					}
					catch (std::runtime_error& e)
					{
						cerr << e.what() << std::endl;
						break;
					}

					execCommunication.init(interpreter.retActualCommandParams());
					if (interpreter.retActualCommandParams().family == Family::P4xx)
					{
						const uint8_t loopInitialization = 20;
						bool InitP432 = false;

						for (uint8_t i = 0; i < loopInitialization; i++)
						{
							execCommunication.transmitBufferInit432();
							ptrBigBuf = execCommunication.receiveBuffer(1, execCommand.retVerbose());
							if (*ptrBigBuf == 0x00)
							{
								InitP432 = true;
								break;
							}
						}
						if (InitP432 == false)
						{
							std::cout << "Initialization of BSL P432 failed... Exit the scripter..." << std::endl;
							return(-1);
						}
						else
						{
							std::cout << "Initialization of BSL P432 succeed..." << std::endl;
						}
					}
				}
				else
				{
					interpreter.matchCommand();
					//*******************************************************//
					// RX - READING commands
					//*******************************************************//
					if ((interpreter.retActualCommandParams().command == Command::RX_DATA_BLOCK) ||
						(interpreter.retActualCommandParams().command == Command::RX_DATA_BLOCK_32) ||
						(interpreter.retActualCommandParams().command == Command::RX_PASSWORD) ||
						(interpreter.retActualCommandParams().command == Command::RX_PASSWORD_32) ||
						(interpreter.retActualCommandParams().command == Command::RX_DATA_BLOCK_FAST)
						)
					{
						
						execCommunication.resetBytesWritten();

						if (interpreter.retActualCommandParams().paramsInput[1] != "")
						{
							execCommunication.resetBytesWritten();

							bool initReadSucceed = true;

							try
							{
								scripterStream.initReadTxtFile(interpreter.retActualCommandParams(), mainPathDir);
							}
							catch (std::runtime_error& e)
							{
								cerr << e.what() << endl;
								initReadSucceed = false;
							}
							
							if (initReadSucceed)
							{
								while (scripterStream.retReadStatus())
								{
									scripterStream.readTxtFile();

									if (scripterStream.retSendStatus())
									{
										execCommand.setDataBlock(scripterStream.retDataBlockRead());
										execCommand.callFunc(interpreter.retActualCommandParams());

										if (interpreter.retActualCommandParams().command == Command::RX_DATA_BLOCK_FAST)
										{
											execCommunication.transmitBuffer(execCommand.retDataBuffer(), execCommand.retVerbose());
										}
										else
										{
											execCommunication.transmitBuffer(execCommand.retDataBuffer(), execCommand.retVerbose());
											ptrBigBuf = execCommunication.receiveBuffer(execCommand.retNumResponseBytes(), execCommand.retVerbose());
										}

									}
								}
							}
							
						}
						else
						{
							DataBlock passDataBlock;

							if ((interpreter.retActualCommandParams().command == Command::RX_PASSWORD) &&
								(interpreter.retActualCommandParams().family == Family::F543x))
							{
								for (uint16_t i = 0; i < 16; i++)
								{
									passDataBlock.data[i] = ui8Password32[i];
								}
								passDataBlock.startAddress = 0xFFE0;
								passDataBlock.numOfBytes = 16;
							}
							else if (interpreter.retActualCommandParams().command == Command::RX_PASSWORD_32)
							{
								for (uint16_t i = 0; i < 256; i++)
								{
									passDataBlock.data[i] = ui8Password256[i];
								}
								passDataBlock.startAddress = 0;
								passDataBlock.numOfBytes = 256;
							}
							else
							{
								for (uint16_t i = 0; i < 32; i++)
								{
									passDataBlock.data[i] = ui8Password32[i];
									passDataBlock.startAddress = 0xFFE0;
									passDataBlock.numOfBytes = 32;
								}
							}
							execCommand.setDataBlock(passDataBlock);
							execCommand.callFunc(interpreter.retActualCommandParams());
							execCommunication.transmitBuffer(execCommand.retDataBuffer(), execCommand.retVerbose());
							ptrBigBuf = execCommunication.receiveBuffer(execCommand.retNumResponseBytes(), execCommand.retVerbose());
						}

						getResponseString = interpreter.retResponse(ptrBigBuf);
						std::cout << getResponseString << std::endl;
						scripterStream.closeReadTxtFile();

						std::cout << "Bytes sent to the BSL Target device is " << (int)execCommunication.retBytesWritten() << std::endl;
					}
					//*******************************************************//
					// OTHER commands
					//*******************************************************//
					else
					{
						execCommand.callFunc(interpreter.retActualCommandParams());

						if (execCommand.retRunFunc())
						{
							if ((interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK) ||
								(interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK_32))
							{
								execCommunication.resetBytesRead();
							}

							if ((interpreter.retActualCommandParams().command != Command::DELAY) &&
								(interpreter.retActualCommandParams().command != Command::VERBOSE))
							{
								execCommunication.transmitBuffer(execCommand.retDataBuffer(), execCommand.retVerbose());

								ptrBigBuf = execCommunication.receiveBuffer(execCommand.retNumResponseBytes(), execCommand.retVerbose());

								if (interpreter.retActualCommandParams().command == Command::CHANGE_BAUD_RATE)
								{
									interpreter.setupNewBaudRate(execCommand.retNewBaudRate());
									execCommunication.changeBaudRate(execCommand.retNewBaudRate());
								}
								else if ((interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK) ||
									(interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK_32)
									)
								{
									bool initWriteSucceed = false;

									try
									{
										scripterStream.initWriteTxtFile(interpreter.retActualCommandParams(), mainPathDir);
									}
									catch (std::runtime_error& e)
									{
										cerr << e.what() << endl;
									}

									initWriteSucceed = true;

									if (initWriteSucceed)
									{
										scripterStream.writeTxtFile(ptrBigBuf, execCommand.retNumResponseBytes());
										scripterStream.closeWriteTxtFile();
									}
 								}

								getResponseString = interpreter.retResponse(ptrBigBuf);
								std::cout << getResponseString << std::endl;

								if ((interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK) ||
									(interpreter.retActualCommandParams().command == Command::TX_DATA_BLOCK_32))
								{
									std::cout << "Bytes received from the BSL Target device is " << (int)execCommunication.retBytesRead() << std::endl;
								}

								if (((interpreter.retActualCommandParams().command == Command::MASS_ERASE) &&
									 (interpreter.retActualCommandParams().family == Family::FRxx)) || 
									((interpreter.retActualCommandParams().command == Command::REBOOT_RESET) &&
									 (interpreter.retActualCommandParams().family == Family::P4xx))
									)

								{
									execCommunication.close();
								}
							}
						}
					}
				}
			}
		}

	}
	return 0;
}
