/*
BSL-Scripter-3.0.1.0

==FlashFram.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/

#include "stdafx.h"

/*******************************************************************************
*Class:       FramFlash
*Function:    readParameters
*Description: setup the parameters for each BSL command in FramFlash
*Parameters:  ActualCommandParams actualCommandParams:      get command
*Returns:     none
*******************************************************************************/
void FramFlash::readParameters(ActualCommandParams actualCommandParams)
{
	switch (actualCommandParams.command)
	{
	case (Command::CHANGE_BAUD_RATE) :
		newBaudRate = std::stoi(actualCommandParams.paramsInput[1]);
		if (actualCommandParams.protocol == Protocol::UART)
		{
			switch (newBaudRate)
			{
				case 4800:  commandBaudRate = BAUD430_4800;  break;
				case 9600:  commandBaudRate = BAUD430_9600;  break;
				case 19200: commandBaudRate = BAUD430_19200; break;
				case 38400: commandBaudRate = BAUD430_38400; break;
				case 57600: commandBaudRate = BAUD430_57600; break;
				case 115200:commandBaudRate = BAUD430_115200; break;
				default:throw runtime_error("Wrong baud rate setting for UART communication..."); break;
			}
		}
		else
		{
			throw("CHANGE_BAUD_RATE command is not available for I2C and SPI protocol...");
		}
		break;

	case (Command::DELAY) :
		delayMs = std::stoi(actualCommandParams.paramsInput[1]);
		break;

	case (Command::CRC_CHECK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);
		break;

	case (Command::ERASE_SEGMENT) :
		if ((actualCommandParams.family == Family::F5xx) || 
			(actualCommandParams.family == Family::F6xx) ||
			(actualCommandParams.family == Family::F543x)
		   )
		{
			actualCommandParams.paramsInput[1].erase(0, 2);
			address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		}
		else
		{
			throw ("ERASE_SEGMENT command is not available for other than Flash Families...");
		}
		break;

	case (Command::MASS_ERASE) :
		if (actualCommandParams.family == Family::FRxx)
		{
			minResponseBytes = 8;
		}
		else
		{
			minResponseBytes = 0;
		}
		break;

	case (Command::SET_PC) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::RX_PASSWORD) :
		if (actualCommandParams.family == Family::F543x)
		{
			numPass = PASS_16;
		}
		else
		{
			numPass = PASS_32;
		}

        if (actualCommandParams.paramsInput[1] == "")
		{
			selPass = PASS_DEFAULT;
		}
		else
		{
			selPass = PASS_FILE;
		}

		break;

	case (Command::TX_DATA_BLOCK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);
		
		totalBytesReceived = 0;
		uint32_t totalBytesLeft;

		if (actualCommandParams.protocol == Protocol::USB)
		{
			const uint8_t  numHeaderUsb1 = 3; //3
			const uint8_t  numHeaderUsb2 = 3; //3
			const uint32_t maxBufSizeUsb = 64;
			const uint32_t maxDataReceiveUsb1 = maxBufSizeUsb - numHeaderUsb1 + 3;//first packet
			const uint32_t maxDataReceiveUsb2 = maxBufSizeUsb - numHeaderUsb2 + 3;


			if (length > maxDataReceiveUsb1)
			{
				totalBytesReceived = maxBufSizeUsb;
				totalBytesLeft = length - maxDataReceiveUsb1;

				while (totalBytesLeft > 0)
				{
					if (totalBytesLeft < maxDataReceiveUsb1)
					{
						//last package
						totalBytesReceived += (totalBytesLeft + numHeaderUsb2) + 4;
						totalBytesLeft = 0;
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSizeUsb + numHeaderUsb2;
						totalBytesLeft -= maxDataReceiveUsb2;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeaderUsb1 + 4;
			}
		}
		else
		{
			if (length > maxDataReceive1Uart)
			{
				totalBytesReceived = maxBufSizeUart;
				totalBytesLeft = length - maxDataReceive1Uart;

				while (totalBytesLeft > 0)
				{ 
					if (totalBytesLeft < maxDataReceive1Uart)
					{
						//last package
						totalBytesReceived += (totalBytesLeft + numHeader2Uart );
						totalBytesLeft = 0;
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSizeUart;
						totalBytesLeft -= maxDataReceive2Uart;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeader1Uart;
			}
		}
		break;

	default:
		//no reading parameters
		break;
	}
}

/*******************************************************************************
*Class:       FramFlash
*Function:    changeBaudRate
*Description: Execution of CHANGE_BAUD_RATE in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::changeBaudRate()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = CHANGE_BAUD_RATE;
	commandBuffer.data[1] = commandBaudRate;

	commandBuffer.size = 2;

	numResponseBytes = 1;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    crcCheck32
*Description: Execution of CRC_CHECK_32 in FramFlash family
*Parameters:  none
*Returns:     exception 
*******************************************************************************/
DataBuffer FramFlash::crcCheck32()
{
	throw runtime_error("CRC_Check_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    crcCheck
*Description: Execution of CRC_CHECK in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::crcCheck()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = CRC_CHECK;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)(length & 0xFF);
	commandBuffer.data[5] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 6;

	numResponseBytes = 9;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    delay
*Description: Execution of DELAY in FramFlash family
*Parameters:  none
*Returns:     none
*******************************************************************************/
void FramFlash::delay()
{
	std::this_thread::sleep_for(std::chrono::milliseconds(delayMs));
}

/*******************************************************************************
*Class:       FramFlash
*Function:    eraseCheck
*Description: Execution of ERASE_CHECK in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::eraseCheck()
{
	throw runtime_error("ERASE_CHECK command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    eraseMainOrInfo
*Description: Execution of ERASE_MAIN_OR_INFO in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::eraseMainOrInfo()
{
	throw runtime_error("ERASE_MAIN_OR_INFO command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    eraseSegment32
*Description: Execution of ERASE_SEGMENT_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::eraseSegment32()
{
	throw runtime_error("ERASE_SEGMENT_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    eraseSegment
*Description: Execution of ERASE_SEGMENT in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::eraseSegment()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ERASE_SEGMENT;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);

	commandBuffer.size = 4;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    setPc32
*Description: Execution of SET_PC_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::setPc32()
{
	throw runtime_error("SET_PC_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    setPc
*Description: Execution of SET_PC in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::setPc()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = SET_PC;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);

	commandBuffer.size = 4;
 
	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    massErase
*Description: Execution of MASS_ERASE in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::massErase()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = MASS_ERASE;

	commandBuffer.size = 1;

	numResponseBytes = 8 - minResponseBytes;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rebootReset
*Description: Execution of REBOOT_RESET in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::rebootReset()
{
	throw runtime_error("REBOOT_RESET command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rxDataBlockFast
*Description: Execution of RX_DATA_BLOCK_FAST in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::rxDataBlockFast()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_DATA_BLOCK_FAST;
	commandBuffer.data[1] = (uint8_t)(dataBlock.startAddress & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress >> 16) & 0xFF);

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 4] = dataBlock.data[i];
	}

	commandBuffer.size = dataBlock.numOfBytes + 4;

	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rxDataBlock32
*Description: Execution of RX_DATA_BLOCK_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::rxDataBlock32()
{
	throw runtime_error("RX_DATA_BLOCK_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rxDataBlock
*Description: Execution of RX_DATA_BLOCK in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::rxDataBlock()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_DATA_BLOCK;
	commandBuffer.data[1] = (uint8_t)(dataBlock.startAddress & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress >> 16) & 0xFF);

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 4] = dataBlock.data[i];
	}
	commandBuffer.size = dataBlock.numOfBytes + 4;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rxPassword32
*Description: Execution of RX_PASSWORD_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::rxPassword32()
{
	throw runtime_error("RX_PASSWORD_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    rxPassword
*Description: Execution of RX_PASSWORD in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::rxPassword()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_PASSWORD;

	if (selPass == PASS_FILE)
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = dataBlock.data[i];
		}
	}
	else
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = ui8Password32[i];
		}
	}

	commandBuffer.size = 1 + numPass;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    setMemoryOffset
*Description: Execution of SET_MEM_OFFSET in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::setMemoryOffset()
{

	throw runtime_error("SET_MEMORY_OFFSET command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    toggleInfo
*Description: Execution of TOGGLE_INFO in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::toggleInfo()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TOGGLE_INFO;
	commandBuffer.size = 1;

	numResponseBytes = 11;  //uart 8 usb 11

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    txDataBlock32
*Description: Execution of TX_DATA_BLOCK_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::txDataBlock32()
{
	throw runtime_error("TX_DATA_BLOCK_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    txDataBlock
*Description: Execution of TX_DATA_BLOCK in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::txDataBlock()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_DATA_BLOCK;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)(length & 0xFF);
	commandBuffer.data[5] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 6;

	numResponseBytes = totalBytesReceived;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    bslVersion32
*Description: Execution of TX_BSL_VERSION_32 in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer FramFlash::bslVersion32()
{
	throw runtime_error("BSL_VERSION_32 command is not available for Flash-Fram families...");
}

/*******************************************************************************
*Class:       FramFlash
*Function:    bslVersion
*Description: Execution of TX_BSL_VERSION in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::bslVersion()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_BSL_VERSION;
	commandBuffer.size = 1;

	numResponseBytes = 11;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    txBufferSize
*Description: Execution of TX_BUFFER_SIZE in FramFlash family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer FramFlash::txBufferSize()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_BUFFER_SIZE;
	commandBuffer.size = 1;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    initVerbose
*Description: set the verbose mode to off as initialization
*Parameters:  none
*Returns:     none
*******************************************************************************/
void FramFlash::initVerbose()
{
	verboseToggle = false;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    verbose
*Description: toggle verbose mode
*Parameters:  none
*Returns:     none
*******************************************************************************/
void FramFlash::verbose()
{
	if (verboseToggle == false)
	{
		verboseToggle = true;
	}
	else
	{
		verboseToggle = false;
	}

}

/*******************************************************************************
*Class:       FramFlash
*Function:    retVerbose
*Description: return verbose mode
*Parameters:  none
*Returns:     bool true : verbose on
*             bool false: verbose off
*******************************************************************************/
bool FramFlash::retVerbose()
{
	return verboseToggle;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    retNewBaudRate
*Description: return baud rate value
*Parameters:  none
*Returns:     uint32_t newBaudRate : new baud rate value
*******************************************************************************/
uint32_t FramFlash::retNewBaudRate()
{
	return newBaudRate;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    retNumResponseBytes
*Description: return number of response byte of respective command
*Parameters:  none
*Returns:     uint32_t numResponseBytes :umber of response byte
*******************************************************************************/
uint32_t FramFlash::retNumResponseBytes()
{
	return numResponseBytes;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    setDataBlock
*Description: copy DataBlock from streaming input to be applied in BSL command
*Parameters:  DataBlock dataBlockStream: input DataBlock from streaming input
*Returns:     none
*******************************************************************************/
void FramFlash::setDataBlock(DataBlock dataBlockStream)
{
	dataBlock = dataBlockStream;
}
