#pragma once

#include "stdafx.h"

//
// INTERPRETER
//



class Interpreter
{
public:
	Interpreter(); //Constructor
	void clearParamsInput();
	bool checkCommentLine(std::string command);
	void checkModeParameters();

	void interpretCommandParams(string command);
	void matchCommand();

	ActualCommandParams retActualCommandParams();
	void setupNewBaudRate(uint32_t newBaudRate);

	std::string retResponse(uint8_t *buf);

	//some return response
	uint16_t retCrcBsl();
	bool retResetComm();
	void clrResetComm();

private:

	//functions
	std::string intToHex(uint8_t input);
	std::string intToHexCrc(uint16_t input);
	std::string intToString(uint8_t input);
	//
	std::string familyTable[11];
	std::string protocolTable[4];
	ActualCommandParams actualCommandParams;
	CommandParams commandPar[25];
	
	uint16_t  crcBsl;
	uint16_t  bslVersion[5];
	uint16_t  bufferSize;
	bool resetComm;

	//responseHandler

	std::string changeBaudRate(uint8_t *buf);
	std::string crcCheck(uint8_t *buf);
	//std::string eraseSegment(uint8_t *buf);
	std::string setPC(uint8_t *buf);
	std::string massErase(uint8_t *buf);
	std::string rxDataBlockFast(uint8_t *buf);
	std::string rxDataBlock(uint8_t *buf);
	//std::string rxDataBlock32(uint8_t *buf);
	std::string rxPassword(uint8_t *buf);
	//std::string toggleInfo(uint8_t *buf);
	std::string txDataBlock(uint8_t *buf);
	std::string txBslVersion32(uint8_t *buf);
	std::string txBslVersion(uint8_t *buf);
	std::string txBufferSize(uint8_t *buf);
	std::string unknownCommand(uint8_t *buf);

	//Response byte from
	const uint8_t HEADER_BYTE = 0x80;
	const uint8_t USB_HEADER_BYTE = 0x3F;
	const uint8_t BSL_DATA_REPLY = 0x3A;
	const uint8_t BSL_MESSAGE_REPLY = 0x3B;
	const uint8_t FLASH_ACK = 0x00;
	
	// Error Codes from the API
	const uint8_t SUCCESSFUL_OPERATION = 0x00;
	const uint8_t NO_EXCEPTIONS = 0x00;
	const uint8_t FLASH_WRITE_CHECK_FAILED = 0x01;
	const uint8_t FLASH_FAIL_BIT_SET = 0x02;
	const uint8_t VOLTAGE_CHANGE_DURING_PROGRAM = 0x03;
	const uint8_t BSL_LOCKED = 0x04;
	const uint8_t BSL_PASSWORD_ERROR = 0x05;
	const uint8_t BYTE_WRITE_FORBIDDEN = 0x06;
	const uint8_t BYTE_UNKNOWN_COMMAND = 0x07;

	std::string checkHeader(uint8_t *bufHeader, uint8_t* messageReply, uint8_t *coreResponse);
};

