/*
BSL-Scripter-3.0.1.0

==P432.cpp==

Copyright (c) 2008  2015 Texas Instruments Incorporated

All rights reserved not granted herein.

Limited License.

Texas Instruments Incorporated grants a world-wide, royalty-free,
non-exclusive license under copyrights and patents it now or
hereafter owns or controls to make, have made, use, import,
offer to sell and sell ("Utilize") this software subject to
the terms herein.  With respect to the foregoing patent license,
such license is granted  solely to the extent that any such patent
is necessary to Utilize the software alone.  The patent license
shall not apply to any combinations which include this software,
other than combinations with devices manufactured by or for TI
(TI Devices). No hardware patent is licensed hereunder.

Redistributions must preserve existing copyright notices and
reproduce this license (including the above copyright notice
and the disclaimer and (if applicable) source code license
limitations below) in the documentation and/or other materials
provided with the distribution

Redistribution and use in binary form, without modification,
are permitted provided that the following conditions are met:

*	No reverse engineering, decompilation, or disassembly of
this software is permitted with respect to any software
provided in binary form.

*	any redistribution and use are licensed by TI for use only
with TI Devices.

*	Nothing shall obligate TI to provide you with source code
for the software licensed and provided to you in object code.

If software source code is provided to you, modification and
redistribution of the source code are permitted provided that
the following conditions are met:

*	any redistribution and use of the source code, including any
resulting derivative works, are licensed by TI for use only
with TI Devices.

*	any redistribution and use of any object code compiled from
the source code and any resulting derivative works, are licensed
by TI for use only with TI Devices.

Neither the name of Texas Instruments Incorporated nor the names
of its suppliers may be used to endorse or promote products derived
from this software without specific prior written permission.

DISCLAIMER.

THIS SOFTWARE IS PROVIDED BY TI AND TIS LICENSORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL TI AND TIS LICENSORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

Version:	Release:		Author:
3.0.1.0		Aug/07/2015		Fatmawati Santosa

*/
#include "stdafx.h"

/*******************************************************************************
*Class:       P432
*Function:    readParameters
*Description: setup the parameters for each BSL command in P432
*Parameters:  ActualCommandParams actualCommandParams:      get command
*Returns:     none
*******************************************************************************/
void P432::readParameters(ActualCommandParams actualCommandParams)
{
	switch (actualCommandParams.command)
	{
	case (Command::CHANGE_BAUD_RATE) :
		newBaudRate = std::stoi(actualCommandParams.paramsInput[1]);
		if (actualCommandParams.protocol == Protocol::UART)
		{
			switch (newBaudRate)
			{
			case 4800:  commandBaudRate = BAUD430_4800;  break;
			case 9600:  commandBaudRate = BAUD430_9600;  break;
			case 19200: commandBaudRate = BAUD430_19200; break;
			case 38400: commandBaudRate = BAUD430_38400; break;
			case 57600: commandBaudRate = BAUD430_57600; break;
			case 115200:commandBaudRate = BAUD430_115200; break;
			default:throw runtime_error("Wrong baud rate setting for UART communication..."); break;
			}
		}
		else
		{
			throw("CHANGE_BAUD_RATE command is not available for I2C and SPI protocol...");
		}
		break;

	case (Command::DELAY) :
		delayMs = std::stoi(actualCommandParams.paramsInput[1]);
		break;

	case (Command::CRC_CHECK_32) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address32 = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);
		break;

	case (Command::CRC_CHECK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);
		break;

	case (Command::ERASE_SEGMENT_32) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address32 = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::ERASE_SEGMENT) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::SET_PC_32) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address32 = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::SET_PC) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);
		break;

	case (Command::RX_PASSWORD) :
		if (actualCommandParams.family == Family::F543x)
		{
		numPass = PASS_16;
		}
		else
		{
			numPass = PASS_32;
		}

		if (actualCommandParams.paramsInput[1] == "")
		{
			selPass = PASS_DEFAULT;
		}
		else
		{
			selPass = PASS_FILE;
		}

		break;


	case (Command::RX_PASSWORD_32) :

		numPass = 256;

		if (actualCommandParams.paramsInput[1] == "")
		{
			selPass = PASS_DEFAULT;
		}
		else
		{
			selPass = PASS_FILE;
		}
		
		break;

	case (Command::TX_DATA_BLOCK_32) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address32 = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);

		totalBytesReceived = 0;
		uint32_t totalBytesLeft32;

		if (actualCommandParams.protocol == Protocol::USB)
		{
			uint32_t maxBufSize = 48;
			uint32_t maxDataReceive1 = 48 - 3;//first packet
			uint32_t maxDataReceive2 = 48 - 3;
			uint8_t  numHeader1 = 3;
			uint8_t  numHeader2 = 3;

			if (length > maxDataReceive1)
			{
				totalBytesReceived = maxBufSize;
				totalBytesLeft32 = length - maxDataReceive1;

				while (totalBytesLeft32 > 0)
				{
					if (totalBytesLeft32 < maxDataReceive2)
					{
						//last package
						totalBytesReceived += (totalBytesLeft32 + numHeader2);
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSize;
						totalBytesLeft32 -= maxDataReceive2;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeader1;
			}
		}
		else
		{
			uint32_t maxBufSize = 266;
			uint32_t maxDataReceive1 = 259;//first packet
			uint32_t maxDataReceive2 = 260;
			uint8_t  numHeader1 = 7;
			uint8_t  numHeader2 = 6;

			if (length > maxDataReceive1)
			{
				totalBytesReceived = maxBufSize;
				totalBytesLeft32 = length - maxDataReceive1;

				while (totalBytesLeft32 > 0)
				{
					if (totalBytesLeft32 < maxDataReceive2)
					{
						//last package
						totalBytesReceived += (totalBytesLeft32 + numHeader2);
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSize;
						totalBytesLeft32 -= maxDataReceive2;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeader1;
			}
		}

		break;

	case (Command::TX_DATA_BLOCK) :
		actualCommandParams.paramsInput[1].erase(0, 2);
		address = std::stoul(actualCommandParams.paramsInput[1], nullptr, 16);

		actualCommandParams.paramsInput[2].erase(0, 2);
		length = std::stoul(actualCommandParams.paramsInput[2], nullptr, 16);

		totalBytesReceived = 0;
		uint32_t totalBytesLeft;

		if (actualCommandParams.protocol == Protocol::USB)
		{
			uint32_t maxBufSize = 48;
			uint32_t maxDataReceive1 = 48 - 3;//first packet
			uint32_t maxDataReceive2 = 48 - 3;
			uint8_t  numHeader1 = 3;
			uint8_t  numHeader2 = 3;

			if (length > maxDataReceive1)
			{
				totalBytesReceived = maxBufSize;
				totalBytesLeft = length - maxDataReceive1;

				while (totalBytesLeft > 0)
				{
					if (totalBytesLeft < maxDataReceive2)
					{
						//last package
						totalBytesReceived += (totalBytesLeft + numHeader2);
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSize;
						totalBytesLeft -= maxDataReceive2;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeader1;
			}
		}
		else
		{
			uint32_t maxBufSize = 266;
			uint32_t maxDataReceive1 = 259;//first packet
			uint32_t maxDataReceive2 = 260;
			uint8_t  numHeader1 = 7;
			uint8_t  numHeader2 = 6;

			if (length > maxDataReceive1)
			{
				totalBytesReceived = maxBufSize;
				totalBytesLeft = length - maxDataReceive1;

				while (totalBytesLeft > 0)
				{
					if (totalBytesLeft < maxDataReceive2)
					{
						//last package
						totalBytesReceived += (totalBytesLeft + numHeader2);
						break;
					}
					else
					{
						//still full package
						totalBytesReceived += maxBufSize;
						totalBytesLeft -= maxDataReceive2;
					}
				}
			}
			else
			{
				totalBytesReceived = length + numHeader1;
			}
		}

		break;

	default:
		//no reading parameters
		break;
	}
}

/*******************************************************************************
*Class:       P432
*Function:    changeBaudRate
*Description: Execution of CHANGE_BAUD_RATE in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::changeBaudRate()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = CHANGE_BAUD_RATE;
	commandBuffer.data[1] = commandBaudRate;

	commandBuffer.size = 2;

	numResponseBytes = 1;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    crcCheck32
*Description: Execution of CRC_CHECK_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::crcCheck32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = CRC_CHECK_32;
	commandBuffer.data[1] = (uint8_t)(address32 & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address32 >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address32 >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address32 >> 24) & 0xFF);
	commandBuffer.data[5] = (uint8_t)(length & 0xFF);
	commandBuffer.data[6] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 7;

	numResponseBytes = 9;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    crcCheck
*Description: Execution of CRC_CHECK in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::crcCheck()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = CRC_CHECK;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)(length & 0xFF);
	commandBuffer.data[5] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 6;

	numResponseBytes = 9;

	return commandBuffer;

}

/*******************************************************************************
*Class:       P432
*Function:    delay
*Description: Execution of DELAY in P432 family
*Parameters:  none
*Returns:     none
*******************************************************************************/
void P432::delay()
{
	std::this_thread::sleep_for(std::chrono::milliseconds(delayMs));
}

/*******************************************************************************
*Class:       P432
*Function:    eraseCheck
*Description: Execution of ERASE_CHECK in P432 family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer P432::eraseCheck()
{
	throw runtime_error("ERASE_CHECK command is not available for P432 families...");
}

/*******************************************************************************
*Class:       P432
*Function:    eraseMainOrInfo
*Description: Execution of ERASE_MAIN_OR_INFO in P432 family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer P432::eraseMainOrInfo()
{
	throw runtime_error("ERASE_MAIN_OR_INFO command is not available for P432 families...");
}

/*******************************************************************************
*Class:       P432
*Function:    eraseSegment32
*Description: Execution of ERASE_SEGMENT_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::eraseSegment32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ERASE_SEGMENT_32;
	commandBuffer.data[1] = (uint8_t)(address32 & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address32 >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address32 >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address32 >> 24) & 0xFF);

	commandBuffer.size = 5;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    eraseSegment
*Description: Execution of ERASE_SEGMENT in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::eraseSegment()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = ERASE_SEGMENT;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);

	commandBuffer.size = 4;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    setPc32
*Description: Execution of SET_PC_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::setPc32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = SET_PC_32;
	commandBuffer.data[1] = (uint8_t)(address32 & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address32 >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address32 >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address32 >> 24) & 0xFF);

	commandBuffer.size = 5;


	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    setPc
*Description: Execution of SET_PC in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::setPc()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = SET_PC;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);

	commandBuffer.size = 4;


	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    massErase
*Description: Execution of MASS_ERASE in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::massErase()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = MASS_ERASE;

	commandBuffer.size = 1;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rebootReset
*Description: Execution of REBOOT_RESET in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::rebootReset()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = REBOOT_RESET;

	commandBuffer.size = 1;

	numResponseBytes = 0;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rxDataBlockFast
*Description: Execution of RX_DATA_BLOCK_FAST in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::rxDataBlockFast()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_DATA_BLOCK_FAST;
	commandBuffer.data[1] = (uint8_t)(dataBlock.startAddress & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress >> 16) & 0xFF);

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 4] = dataBlock.data[i];
	}

	commandBuffer.size = dataBlock.numOfBytes + 4;

	numResponseBytes = 1;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rxDataBlock32
*Description: Execution of RX_DATA_BLOCK_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::rxDataBlock32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_DATA_BLOCK_32;

	commandBuffer.data[1] = (uint8_t)(dataBlock.startAddress & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((dataBlock.startAddress >> 24) & 0xFF);

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 5] = dataBlock.data[i];
	}

	commandBuffer.size = 5 + dataBlock.numOfBytes;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rxDataBlock
*Description: Execution of RX_DATA_BLOCK in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::rxDataBlock()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_DATA_BLOCK;
	commandBuffer.data[1] = (uint8_t)(dataBlock.startAddress & 0xFF);
	commandBuffer.data[2] = (uint8_t)((dataBlock.startAddress >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((dataBlock.startAddress >> 16) & 0xFF);

	for (uint16_t i = 0; i < dataBlock.numOfBytes; i++)
	{
		commandBuffer.data[i + 4] = dataBlock.data[i];
	}
	commandBuffer.size = dataBlock.numOfBytes + 4;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rxPassword32
*Description: Execution of RX_PASSWORD_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::rxPassword32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_PASSWORD_32;

	if (selPass == PASS_FILE)
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = dataBlock.data[i];
		}
	}
	else
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = ui8Password256[i];
		}
	}

	commandBuffer.size = 1 + numPass;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    rxPassword
*Description: Execution of RX_PASSWORD in P432 family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer P432::rxPassword()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = RX_PASSWORD;

	if (selPass == PASS_FILE)
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = dataBlock.data[i];
		}
	}
	else
	{
		for (uint16_t i = 0; i < numPass; i++)
		{
			commandBuffer.data[i + 1] = ui8Password32[i];
		}
	}

	commandBuffer.size = 1 + numPass;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       FramFlash
*Function:    setMemoryOffset
*Description: Execution of SET_MEM_OFFSET in FramFlash family
*Parameters:  none
*Returns:     exception
*******************************************************************************/
DataBuffer P432::setMemoryOffset()
{

	throw runtime_error("SET_MEMORY_OFFSET command is not available for P432 families...");
}

/*******************************************************************************
*Class:       P432
*Function:    toggleInfo
*Description: Execution of TOGGLE_INFO in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/

DataBuffer P432::toggleInfo()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TOGGLE_INFO;
	commandBuffer.size = 1;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    txDataBlock32
*Description: Execution of TX_DATA_BLOCK_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::txDataBlock32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_DATA_BLOCK_32;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)((address >> 24) & 0xFF);
	commandBuffer.data[5] = (uint8_t)(length & 0xFF);
	commandBuffer.data[6] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 7;

	numResponseBytes = totalBytesReceived;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    txDataBlock
*Description: Execution of TX_DATA_BLOCK in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::txDataBlock()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_DATA_BLOCK;
	commandBuffer.data[1] = (uint8_t)(address & 0xFF);
	commandBuffer.data[2] = (uint8_t)((address >> 8) & 0xFF);
	commandBuffer.data[3] = (uint8_t)((address >> 16) & 0xFF);
	commandBuffer.data[4] = (uint8_t)(length & 0xFF);
	commandBuffer.data[5] = (uint8_t)((length >> 8) & 0xFF);

	commandBuffer.size = 6;

	numResponseBytes = totalBytesReceived;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    bslVersion32
*Description: Execution of TX_BSL_VERSION_32 in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::bslVersion32()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_BSL_VERSION_32;

	commandBuffer.size = 1;

	numResponseBytes = 17;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    bslVersion
*Description: Execution of TX_BSL_VERSION in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::bslVersion()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_BSL_VERSION;
	commandBuffer.size = 1;

	numResponseBytes = 17;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    txBufferSize
*Description: Execution of TX_BUFFER_SIZE in P432 family
*Parameters:  none
*Returns:     DataBuffer commandBuffer: BSL command to be transmitted
*******************************************************************************/
DataBuffer P432::txBufferSize()
{
	DataBuffer commandBuffer;

	commandBuffer.data[0] = TX_BUFFER_SIZE;
	commandBuffer.size = 1;

	numResponseBytes = 8;

	return commandBuffer;
}

/*******************************************************************************
*Class:       P432
*Function:    initVerbose
*Description: set the verbose mode to off as initialization
*Parameters:  none
*Returns:     none
*******************************************************************************/
void P432::initVerbose()
{
	verboseToggle = false;
}

/*******************************************************************************
*Class:       P432
*Function:    verbose
*Description: toggle verbose mode
*Parameters:  none
*Returns:     none
*******************************************************************************/
void P432::verbose()
{
	if (verboseToggle == false)
	{
		verboseToggle = true;
	}
	else
	{
		verboseToggle = false;
	}

}

/*******************************************************************************
*Class:       P432
*Function:    retVerbose
*Description: return verbose mode
*Parameters:  none
*Returns:     bool true : verbose on
*             bool false: verbose off
*******************************************************************************/
bool P432::retVerbose()
{
	return verboseToggle;
}

/*******************************************************************************
*Class:       P432
*Function:    retNewBaudRate
*Description: return baud rate value
*Parameters:  none
*Returns:     uint32_t newBaudRate : new baud rate value
*******************************************************************************/
uint32_t P432::retNewBaudRate()
{
	return newBaudRate;
}

/*******************************************************************************
*Class:       P432
*Function:    retNumResponseBytes
*Description: return number of response byte of respective command
*Parameters:  none
*Returns:     uint32_t numResponseBytes :umber of response byte
*******************************************************************************/
uint32_t P432::retNumResponseBytes()
{
	return numResponseBytes;
}

/*******************************************************************************
*Class:       P432
*Function:    setDataBlock
*Description: copy DataBlock from streaming input to be applied in BSL command
*Parameters:  DataBlock dataBlockStream: input DataBlock from streaming input
*Returns:     none
*******************************************************************************/
void P432::setDataBlock(DataBlock dataBlockStream)
{
	dataBlock = dataBlockStream;
}

