#pragma once

#include "stdafx.h"

class Rom : public InterfaceTarget
{
	void readParameters(ActualCommandParams actualCommandParams) override;

	DataBuffer changeBaudRate() override;
	DataBuffer crcCheck32() override;
	DataBuffer crcCheck() override;
	void       delay() override;
	DataBuffer eraseCheck() override;
	DataBuffer eraseMainOrInfo() override;
	DataBuffer eraseSegment32() override;
	DataBuffer eraseSegment() override;
	DataBuffer setPc32() override;
	DataBuffer setPc() override;
	DataBuffer massErase() override;
	DataBuffer rebootReset() override;
	DataBuffer rxDataBlockFast() override;
	DataBuffer rxDataBlock32() override;
	DataBuffer rxDataBlock() override;
	DataBuffer rxPassword32() override;
	DataBuffer rxPassword() override;
	DataBuffer setMemoryOffset() override;
	DataBuffer toggleInfo() override;
	DataBuffer txDataBlock32() override;
	DataBuffer txDataBlock() override;
	DataBuffer bslVersion32() override;
	DataBuffer bslVersion() override;
	DataBuffer txBufferSize() override;
	void       initVerbose() override;
	void       verbose() override;
	bool       retVerbose() override;

	uint32_t   retNewBaudRate() override;
	uint32_t   retNumResponseBytes() override;
	void       setDataBlock(DataBlock dataBlockStream) override;

private:

	//Header byte BSL commands
	const uint8_t ROM_ACK = 0x90;
	const uint8_t RX_PASSWORD = 0x10;
	const uint8_t RX_DATA_BLOCK = 0x12;
	const uint8_t TX_DATA_BLOCK = 0x14;
	const uint8_t ERASE_SEGMENT = 0x16;
	const uint8_t ERASE_MAIN_OR_INFO = 0x16;
	const uint8_t MASS_ERASE = 0x18;
	const uint8_t ERASE_CHECK = 0x1C;
	const uint8_t SET_MEM_OFFSET = 0x21;
	const uint8_t DUMMY_DATA = 0x00;

	//Return response byte
	uint32_t numResponseBytes;

	//read parameters
	uint32_t address;
	uint32_t length;
	uint32_t delayMs;

	uint8_t  numPass;
	const uint8_t PASS_16 = 16;

	uint8_t selPass;
	const uint8_t PASS_DEFAULT = 1;
	const uint8_t PASS_FILE = 2;

	bool verboseToggle;

	uint32_t totalBytesReceived;

	DataBlock dataBlock;
};