#pragma once

#include "stdafx.h"
#include "hidapi.h"
#include "DataType.h"

class UartComm : public InterfaceCommunication
{

public:

	UartComm(uint32_t timeout);

	void init(ActualCommandParams actualCommandParams) override;
	void transmitBufferInit432() override;
	void transmitBuffer(DataBuffer dataBuffer, bool verbose) override;
	uint8_t* receiveBuffer(uint32_t size, bool verbose) override;
	void close() override;

	void changeBaudRate(uint32_t baudRate) override;

	uint32_t   retBytesWritten() override;
	uint32_t   retBytesRead() override;
	void       resetBytesWritten() override;
	void       resetBytesRead() override;

private:

	boost::asio::io_service io_service;
	boost::asio::serial_port port;
	boost::asio::deadline_timer timer;
	const uint32_t timeout = 2000;

	void sendByte(uint8_t *data)
	{
		boost::system::error_code ec;
		boost::asio::write(port, boost::asio::buffer(data, 1), ec);
		std::this_thread::sleep_for(std::chrono::microseconds(100));
	}

	void receiveByte(uint8_t *data)
	{

	}

	void cleanBigBuffer()
	{
		for (uint16_t i=0; i < 20; i++)
		{
			bigBuffer2[i] = 0;
		}
	}

	void cleanInitBuffer()
	{
		for (uint16_t i = 0; i < 4; i++)
		{
			initBuffer[i] = 0;
		}
	}
	bool readError;

	void readComplete(const boost::system::error_code& error,
		uint32_t bytes_transferred)
	{
		timer.cancel();
	}

	void timeOut(const boost::system::error_code& error)
	{
		//Check if the timeout was cancelled
		if (error)
		{
			return;
		}
		port.cancel();
	}

	void invokeBSL();

	uint16_t crc;
	uint8_t  bigBuffer[0xFFFF];
	uint8_t  bigBuffer2[0xFFFF];
	uint8_t  initBuffer[4];

	void checkSumCalculate(uint8_t data)
	{
		uint8_t x;
		x = ((crc >> 8) ^ data) & 0xff;
		x ^= x >> 4;
		crc = (crc << 8) ^ (x << 12) ^ (x << 5) ^ x;
	}

	uint8_t getCheckSumLow()
	{
		return (uint8_t)(crc & 0x00FF);
	}

	uint8_t getCheckSumHigh()
	{
		return (uint8_t)((crc >> 8) & 0x00FF);
	}


	string intToHex(uint8_t input)
	{
		std::stringstream stream;
		stream << std::setfill('0')
			<< std::setw(sizeof(input) * 2)
			<< std::hex
			<< (uint32_t)input;
		return stream.str();
	}

	uint32_t bytesRead;
	uint32_t bytesWrite;

};
